// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import BigInt from 'big-integer';
import Hash from './src/hash.js';
import ElGamal from './src/elGamal.js';
import ElGamalKey from './src/elGamalKey.js';
import SecretShare from './src/secretShare.js';
import tSignFlow from './src/tsignFlow.js';
import tAuthFlow from './src/tauthFlow.js';
import tFastAuthFlow from './src/tfastAuthFlow.js';
import tObliviousFlow from './src/tobliviousFlow.js';
import dAuthFlow from './src/dauthFlow.js';

const p = BigInt('9d39f4f7fc915aded17550bccab12560edf43e03daaaf55847b91b35dcd07bb3', 16);

export default class Index {
    static getKeys(bits = 256, confidence = 32) {
        return ElGamalKey.generate(bits, confidence).map(k => k.toString());
    }

    static getAsmPath() {
        return Hash.asmUrl;
    }

    static setAsmPath(value) {
        Hash.asmUrl = value;
    }

    static seedKeys(pwd) {
        return ElGamalKey.seed(pwd).map(k => k.toString());
    }

    static shareKey(key, number, threshold) {
        return ElGamalKey.from(key).share(number, threshold).map(k => k.toString());
    }

    static shareText(secret, number, threshold) {
        return SecretShare.shareText(secret, number, threshold, p);
    }

    static shareBuffer(secret, number, threshold) {
        return SecretShare.shareBuffer(secret, number, threshold, p);
    }

    static encrypt(text, pub) {
        return ElGamal.encrypt(text, ElGamalKey.from(pub));
    }

    static encryptBuffer(text, pub) {
        return ElGamal.encryptBuffer(text, ElGamalKey.from(pub));
    }

    static decrypt(cypher, prv) {
        return ElGamal.decrypt(cypher, ElGamalKey.from(prv));
    }

    static decryptBuffer(cypher, prv) {
        return ElGamal.decryptBuffer(cypher, ElGamalKey.from(prv));
    }

    static decryptPartial(cypher, prv) {
        return ElGamal.decryptPartial(cypher, ElGamalKey.from(prv));
    }

    static decryptPartialBuffer(cypher, prv) {
        return ElGamal.decryptPartialBuffer(cypher, ElGamalKey.from(prv));
    }

    static combineKeys(partials) {
        const keys = partials.map(k => ElGamalKey.from(k));
        return ElGamalKey.combine(keys).toString();
    }

    static combineDeciphers(partials, pub) {
        return ElGamal.combinePartial(partials, ElGamalKey.from(pub));
    }

    static combinePartialBuffer(partials, pub) {
        return ElGamal.combinePartialBuffer(partials, ElGamalKey.from(pub));
    }

    static combineTexts(partials) {
        return SecretShare.interpolateText(partials, p);
    }

    static interpolateBuffer(partials) {
        return SecretShare.interpolateBuffer(partials, p);
    }
    
    static hashArgon2(pass, salt) {
        return Hash.argon2(pass, salt);
    }
    
    static hashSha(data) {
        return Hash.sha(data);
    }

    static hashPasswords(pass, salt, servers) {
        return Hash.passwords(pass, salt, servers);
    }
    
}

export const getKeys = Index.getKeys;
export const seedKeys = Index.seedKeys;
export const shareKey = Index.shareKey;
export const shareText = Index.shareText;
export const encrypt = Index.encrypt;
export const decrypt = Index.decrypt;
export const decryptPartial = Index.decryptPartial;
export const combineKeys = Index.combineKeys;
export const combineDeciphers = Index.combineDeciphers;
export const combineTexts = Index.combineTexts;
export const hashArgon2 = Index.hashArgon2;
export const hashSha = Index.hashSha;
export const hashPasswords = Index.hashPasswords;
export const getAsmPath = Index.getAsmPath;
export const setAsmPath = Index.setAsmPath;
export const TSignFlow = tSignFlow;
export const TAuthFlow = tAuthFlow;
export const TFastAuthFlow = tFastAuthFlow;
export const TObliviousFlow = tObliviousFlow;
export const DAuthFlow = dAuthFlow;
