// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import BigInt from 'big-integer';
import superagent from 'superagent';
import C25519Point from './c25519Point';
import Hash from './hash';
import AESKey from './aesKey';

export default class DAuthClient {
    /**
     * @param {string} url 
     * @param {string} user 
     */
    constructor(url, user) {
        this.url = url + '/api';
        this.user = user;
    }

    get clientId() {
        return BigInt.fromArray(Array.from(this.clientBuffer), 256, false);
    }

    get clientBuffer() {
        return Hash.shaBuffer((new URL(this.url)).host).slice(0, 16);
    }

    get userId() {
        return BigInt.fromArray(Array.from(this.userBuffer), 256, false);
    }

    get userBuffer() {
        return Hash.shaBuffer(this.user).slice(0, 16);
    }

    /** @param {C25519Point} pass */
    async GetShare(pass) {
        var res = await superagent.get(`${this.url}/dauth/${encodeBase64Url(this.userBuffer)}/share/${encodeBase64Url(pass.toArray())}`);
        return C25519Point.from(Buffer.from(res.text, 'base64'));
    }

    /**
     * @param {Uint8Array} ticks
     * @param {Uint8Array} sign
     */
    async signIn(ticks, sign) {
        var usr = encodeBase64Url(this.userBuffer);
        var tck = encodeBase64Url(ticks);
        var sgn = encodeBase64Url(sign);

        var res = await superagent.get(`${this.url}/dauth/${usr}/signin/${tck}/${sgn}`);
        return Buffer.from(res.text, 'base64');
    }

    /**
     * @param {bigInt.BigInteger} authShare
     * @param {bigInt.BigInteger} keyShare
     * @param {AESKey} secret
     */
    async signUp(authShare, keyShare, secret) {
        var user = encodeBase64Url(this.userBuffer);
        var auth = encodeFromBig(authShare);
        var key = encodeFromBig(keyShare);
        var sec = encodeBase64Url(secret.toString());

        await superagent.post(`${this.url}/dauth/${user}/signup/${auth}/${key}/${sec}`);
    }
}

/** @param {string|Uint8Array} input */
function encodeBase64Url(input) {
    const text = typeof input === "string" ? input : Buffer.from(input).toString('base64');
    return text.replace(/\=/g, "").replace(/\+/g, "-").replace(/\//g, "_");
}

/** @param {bigInt.BigInteger} number */
function encodeFromBig(number) {
    return encodeBase64Url(Buffer.from(number.toArray(256).value));
}
